#version 430 core

uniform mat4 vmMatrix;
uniform mat4 pMatrix;

layout (location = 0) in vec4 inPosition; 
layout (location = 2) in vec2 inUVCoords;
layout (location = 1) in vec3 inNormal;
layout (location = 3) in vec3 inTangent;
layout (location = 4) in vec3 inBitangent;

out VertexData
{
	vec3 positionOS;
	vec3 positionVS;
	vec2 uvCoords;
	vec3 normalOS;
	vec3 tangentOS;
	vec3 bitangentOS;

} v_out;

void main()
{
	vec4 positionOS  = inPosition;
	vec4 positionVS  = vmMatrix * positionOS;
	vec4 positionCS  = pMatrix * positionVS;
	vec3 normalOS    = normalize(inNormal);
	vec3 tangentOS   = normalize(inTangent);
	vec3 bitangentOS = normalize(inBitangent);

	v_out.positionOS  = positionOS.xyz;
	v_out.positionVS  = positionVS.xyz;
	v_out.uvCoords   = inUVCoords;
	v_out.normalOS    = normalOS;

	vec3 tOS = cross(normalOS, bitangentOS);
	vec3 bOS = cross(tangentOS, normalOS);

	if (dot(tOS, tangentOS) < 0)
	{
		tOS = -tOS;
		bOS = -bOS;
	}

	v_out.tangentOS   = tOS;
	v_out.bitangentOS = bOS;

	gl_Position = positionCS;
}