#version 430 core

layout(binding = 0) uniform sampler2D uTexSampler;
layout(binding = 1) uniform sampler2D uDepthSampler;

uniform mat4 vMatrix;
uniform vec3 vPosition;

uniform float near = 0.1;
uniform float far = 1000.0;

uniform float time = 1.5;

uniform vec3 sandColor = vec3(0.85, 0.65, 0.5);
uniform float sandDensity = 1.0;
uniform vec3 sandDirection = vec3(0.5, 0.0, 1.0);

uniform bool mergeBackground = true;

in VertexData
{
	vec2 uvCoords;   
} fs_in;

out vec4 oFragColor;

float hash(vec3 p)
{
	p  = fract(p * vec3(.16532,.17369,.15787));
    p += dot(p.xyz, p.yzx + 19.19);
    return fract(p.x * p.y * p.z);
}

// Value Noise
float noise3D(vec3 pos)
{
	pos *= 10.0;
    float result = 0.0;
    float amplitude = 0.3;
    for (int n = 0; n < 4; n++)
    {
        vec3 i = floor(pos);
        vec3 f = fract(pos); 
        f *= f * (3.0 - 2.0 * f);

		// 3D Interpolation
        result += mix(
            mix(mix(hash(i + vec3(0, 0, 0)),	hash(i + vec3(1, 0, 0)), f.x),
                mix(hash(i + vec3(0, 1, 0)),	hash(i + vec3(1, 1, 0)), f.x),
                f.y),
            mix(mix(hash(i + vec3(0, 0, 1)),	hash(i + vec3(1, 0, 1)), f.x),
                mix(hash(i + vec3(0, 1, 1)),	hash(i + vec3(1, 1, 1)), f.x),
                f.y),
            f.z) * amplitude;

        amplitude *= 0.5;
        pos += pos;
    }
    return result;
}

float sandField(vec3 p, float d)
{
   // Wind
   p += time * sandDirection;
    
   return noise3D(p * 2.2/ (d + 8.0)) * (smoothstep(0.7, 0.0, p.y));
}

vec3 march(vec3 baseColor, vec3 rayOrigin, vec3 rayDirection, float maxDepth)
{
    float distance = 0.5;
    for(int i = 0; i < 7; i++)
    {
        vec3 pos = rayOrigin + rayDirection * distance;
        float accumulatedDensity = sandField(pos, distance);
        baseColor = mix(baseColor, sandColor, clamp(accumulatedDensity * smoothstep(distance, distance * 1.8, maxDepth) * sandDensity, 0.0, 1.0));
        distance *= 1.8;
        if (distance > maxDepth) break;
    }
    return baseColor;
}

void main()
{ 
    vec4 texColor = texture(uTexSampler, fs_in.uvCoords);
    vec4 texDepth = texture(uDepthSampler, fs_in.uvCoords);
	float depth = texDepth.r * 2.0 - 1.0;
	float linearDepth = (2.0 * near * far) / (far + near - depth * (far - near));

	vec2 p = fs_in.uvCoords - 0.5;

	vec3 rayOrigin = vPosition;
	rayOrigin.y *= 0.75;
	rayOrigin.y -= 0.5;
	
    vec3 eyeDir = vec3(0.0, 0.0, -1.0);
    vec3 rightDir = vec3(1.0, 0.0, 0.0);
    vec3 upDir = vec3(0.0, 1.0, 0.0);
	vec3 rayDirection = normalize((p.x * rightDir + p.y * upDir) + eyeDir);
	rayDirection = vec3(vMatrix * vec4(rayDirection, 1.0));

	if (!mergeBackground) {
		texColor = vec4(0.0);
	}

	oFragColor = vec4(march(texColor.rgb, rayOrigin, rayDirection, linearDepth), 1.0);
}